@@ -9,6 +9,8 @@
  *  2. Redistributions in binary form must reproduce the above copyright  
  *     notice, this list of conditions and the following disclaimer in the 
  *     documentation and/or other materials provided with the distribution.
+ *  3. The name of the author may not be used to endorse or promote products
+ *     derived from this software without specific prior written permission.
  *
  *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -23,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_fb.c,v 1.65 2004/11/25 20:03:56 debug Exp $
+ *  $Id: dev_fb.c,v 1.79 2005/01/09 01:55:25 debug Exp $
  *  
  *  Generic framebuffer device.
  *
@@ -44,13 +46,13 @@
  *  TODO:  playstation 2 pixels are stored in another format, actually
  */
 
-#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "console.h"
 #include "devices.h"
+#include "emul.h"
 #include "memory.h"
 #include "misc.h"
 
@@ -61,7 +63,7 @@
 #endif
 
 
-#define	FB_TICK_SHIFT		19
+#define	FB_TICK_SHIFT		18
 
 #define	LOGO_XSIZE		256
 #define	LOGO_YSIZE		256
@@ -360,6 +362,28 @@ void update_framebuffer(struct vfb_data *d, int addr, int len)
 					g = d->framebuffer[fb_addr + 1];
 					b = d->framebuffer[fb_addr + 2];
 					break;
+				/*  TODO: 15 and 16.  */
+				/*  Also copy to the scaledown code below  */
+				case 16:
+					if (d->vfb_type == VFB_HPCMIPS) {
+						b = d->framebuffer[fb_addr] +
+						    (d->framebuffer[fb_addr+1] << 8);
+						r = b >> 11;
+						g = b >> 5;
+						r = r & 31;
+						g = (g & 31) * 2;
+						b = b & 31;
+					} else {
+						r = d->framebuffer[fb_addr] >> 3;
+						g = (d->framebuffer[fb_addr] << 5) +
+						    (d->framebuffer[fb_addr + 1] >> 5);
+						b = d->framebuffer[fb_addr + 1] & 0x1f;
+					}
+
+					r *= 8;
+					g *= 4;
+					b *= 8;
+					break;
 				default:
 					r = g = b = random() & 255;
 				}
@@ -394,19 +418,17 @@ void update_framebuffer(struct vfb_data *d, int addr, int len)
 	if (npixels == 0)
 		npixels = 1;
 
-	for (pixel=0; pixel<npixels; pixel++) {
-		int subx, suby, r, g, b;
-		color_r = color_g = color_b = 0;
-		for (suby=0; suby<scaledown; suby++)
-		    for (subx=0; subx<scaledown; subx++) {
-			int fb_x, fb_y, fb_addr, c;
+	if (d->bit_depth < 8) {
+		for (pixel=0; pixel<npixels; pixel++) {
+			int subx, suby, r, g, b;
+			color_r = color_g = color_b = 0;
+			for (suby=0; suby<scaledown; suby++)
+			    for (subx=0; subx<scaledown; subx++) {
+				int fb_x, fb_y, fb_addr, c;
 
-			fb_x = x * scaledown + subx;
-			fb_y = y * scaledown + suby;
-
-			fb_addr = fb_y * d->xsize + fb_x;
-
-			if (d->bit_depth < 8) {
+				fb_x = x * scaledown + subx;
+				fb_y = y * scaledown + suby;
+				fb_addr = fb_y * d->xsize + fb_x;
 				fb_addr = fb_addr * d->bit_depth;
 				/*  fb_addr is now which _bit_ in framebuffer  */
 
@@ -423,14 +445,57 @@ void update_framebuffer(struct vfb_data *d, int addr, int len)
 				r = d->rgb_palette[c*3 + 0];
 				g = d->rgb_palette[c*3 + 1];
 				b = d->rgb_palette[c*3 + 2];
-			} else if (d->bit_depth == 8) {
-				/*  fb_addr is which _byte_ in framebuffer  */
 
+				color_r += r;
+				color_g += g;
+				color_b += b;
+			    }
+
+			r = color_r / scaledownXscaledown;
+			g = color_g / scaledownXscaledown;
+			b = color_b / scaledownXscaledown;
+			macro_put_pixel();
+			x++;
+		}
+	} else if (d->bit_depth == 8) {
+		for (pixel=0; pixel<npixels; pixel++) {
+			int subx, suby, r, g, b;
+			color_r = color_g = color_b = 0;
+			for (suby=0; suby<scaledown; suby++)
+			    for (subx=0; subx<scaledown; subx++) {
+				int fb_x, fb_y, fb_addr, c;
+
+				fb_x = x * scaledown + subx;
+				fb_y = y * scaledown + suby;
+				fb_addr = fb_y * d->xsize + fb_x;
+				/*  fb_addr is which _byte_ in framebuffer  */
 				c = d->framebuffer[fb_addr] * 3;
 				r = d->rgb_palette[c + 0];
 				g = d->rgb_palette[c + 1];
 				b = d->rgb_palette[c + 2];
-			} else {
+				color_r += r;
+				color_g += g;
+				color_b += b;
+			    }
+
+			r = color_r / scaledownXscaledown;
+			g = color_g / scaledownXscaledown;
+			b = color_b / scaledownXscaledown;
+			macro_put_pixel();
+			x++;
+		}
+	} else {
+		/*  Generic > 8 bit bit-depth:  */
+		for (pixel=0; pixel<npixels; pixel++) {
+			int subx, suby, r, g, b;
+			color_r = color_g = color_b = 0;
+			for (suby=0; suby<scaledown; suby++)
+			    for (subx=0; subx<scaledown; subx++) {
+				int fb_x, fb_y, fb_addr;
+
+				fb_x = x * scaledown + subx;
+				fb_y = y * scaledown + suby;
+				fb_addr = fb_y * d->xsize + fb_x;
 				fb_addr = (fb_addr * d->bit_depth) >> 3;
 				/*  fb_addr is which _byte_ in framebuffer  */
 
@@ -444,20 +509,16 @@ void update_framebuffer(struct vfb_data *d, int addr, int len)
 				default:
 					r = g = b = random() & 255;
 				}
-			}
-
-			color_r += r;
-			color_g += g;
-			color_b += b;
-		    }
-
-		r = color_r / scaledownXscaledown;
-		g = color_g / scaledownXscaledown;
-		b = color_b / scaledownXscaledown;
-
-		macro_put_pixel();
-
-		x++;
+				color_r += r;
+				color_g += g;
+				color_b += b;
+			    }
+			r = color_r / scaledownXscaledown;
+			g = color_g / scaledownXscaledown;
+			b = color_b / scaledownXscaledown;
+			macro_put_pixel();
+			x++;
+		}
 	}
 }
 
@@ -671,7 +732,7 @@ int dev_fb_access(struct cpu *cpu, struct memory *mem,
 	/*
 	 *  If the framebuffer is modified, then we should keep a track
 	 *  of which area(s) we modify, so that the display isn't updated
-	 *  unneccessarily.
+	 *  unnecessarily.
 	 */
 	if (writeflag == MEM_WRITE && cpu->emul->use_x11) {
 		int x, y, x2,y2;
@@ -681,30 +742,6 @@ int dev_fb_access(struct cpu *cpu, struct memory *mem,
 		x2 = ((relative_addr + len) % d->bytes_per_line) * 8 / d->bit_depth;
 		y2 = (relative_addr + len) / d->bytes_per_line;
 
-		/*  Is this far away from the previous updates? Then update:  */
-		if (d->update_y1 != -1) {
-			int diffx1, diffx2, diffy1, diffy2;
-			/*  fhmult may be an integer multiple of the (assumed)
-			    font height, or any other number:  */
-			int short_distance = 8 * 22;
-
-			diffx1 = abs(x - d->update_x1);
-			diffx2 = abs(x - d->update_x2);
-			diffy1 = abs(y - d->update_y1);
-			diffy2 = abs(y - d->update_y2);
-
-			if (diffx2 > diffx1)
-				diffx1 = diffx2;
-			if (diffy2 > diffy1)
-				diffy1 = diffy2;
-
-			/*  use diffx1 and diffy1 from here on  */
-
-			if (diffx1 >= short_distance &&
-			    diffy1 >= short_distance)
-				dev_fb_tick(cpu, d);
-		}
-
 		if (x < d->update_x1 || d->update_x1 == -1)	d->update_x1 = x;
 		if (x > d->update_x2 || d->update_x2 == -1)	d->update_x2 = x;
 
@@ -839,13 +876,15 @@ struct vfb_data *dev_fb_init(struct cpu *cpu, struct memory *mem,
 
 	/*  A nice bootup logo:  */
 	if (logo) {
+		int logo_bottom_margin = LOGO_BOTTOM_MARGIN;
+
 		d->update_x1 = 0;
 		d->update_x2 = LOGO_XSIZE-1;
-		d->update_y1 = d->visible_ysize-LOGO_YSIZE-LOGO_BOTTOM_MARGIN;
-		d->update_y2 = d->visible_ysize-LOGO_BOTTOM_MARGIN;
+		d->update_y1 = d->visible_ysize-LOGO_YSIZE-logo_bottom_margin;
+		d->update_y2 = d->visible_ysize-logo_bottom_margin;
 		for (y=0; y<LOGO_YSIZE; y++)
 			for (x=0; x<LOGO_XSIZE; x++) {
-				int s, a = ((y+d->visible_ysize-LOGO_YSIZE-LOGO_BOTTOM_MARGIN)*d->xsize
+				int s, a = ((y+d->visible_ysize-LOGO_YSIZE-logo_bottom_margin)*d->xsize
 				    + x) * d->bit_depth / 8;
 				int b = fb_logo[(y*LOGO_XSIZE+x) / 8] & (128 >> (x&7));
 				for (s=0; s<d->bit_depth / 8; s++)
