@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -9,6 +9,8 @@
  *  2. Redistributions in binary form must reproduce the above copyright  
  *     notice, this list of conditions and the following disclaimer in the 
  *     documentation and/or other materials provided with the distribution.
+ *  3. The name of the author may not be used to endorse or promote products
+ *     derived from this software without specific prior written permission.
  *
  *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -23,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_pckbc.c,v 1.28 2004/11/20 08:57:13 debug Exp $
+ *  $Id: dev_pckbc.c,v 1.32 2005/01/09 01:55:25 debug Exp $
  *  
  *  Standard 8042 PC keyboard controller, and a 8242WB PS2 keyboard/mouse
  *  controller.
@@ -64,6 +66,7 @@
 
 struct pckbc_data {
 	int		in_use;
+	int		any_command_used;
 
 	int		reg[DEV_PCKBC_LENGTH];
 	int		keyboard_irqnr;
@@ -268,7 +271,7 @@ void dev_pckbc_tick(struct cpu *cpu, void *extra)
 	int port_nr;
 	int ch;
 
-	if (d->in_use && console_charavail()) {
+	if (d->in_use && d->any_command_used && console_charavail()) {
 		ch = console_readchar();
 		if (ch >= 0)
 			ascii_to_pc_scancodes(ch, d);
@@ -297,6 +300,8 @@ static void dev_pckbc_command(struct pckbc_data *d, int port_nr)
 {
 	int cmd = d->reg[PC_CMD];
 
+	d->any_command_used = 1;
+
 	if (d->type == PCKBC_8242)
 		cmd = d->reg[PS2_TXBUF];
 
@@ -363,7 +368,7 @@ int dev_pckbc_access(struct cpu *cpu, struct memory *mem,
 		    (int)relative_addr);
 #endif
 
-	/*  For PICA:  */
+	/*  For JAZZ-based machines:  */
 	if (relative_addr >= 0x60)
 		relative_addr -= 0x60;
 
@@ -603,7 +608,7 @@ void dev_pckbc_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 	}
 	memset(d, 0, sizeof(struct pckbc_data));
 
-	if (type == PCKBC_PICA) {
+	if (type == PCKBC_JAZZ) {
 		type = PCKBC_8042;
 		len = DEV_PCKBC_LENGTH + 0x60;
 	}
