@@ -2,7 +2,7 @@
 #define	DEVICES_H
 
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -12,6 +12,8 @@
  *  2. Redistributions in binary form must reproduce the above copyright  
  *     notice, this list of conditions and the following disclaimer in the 
  *     documentation and/or other materials provided with the distribution.
+ *  3. The name of the author may not be used to endorse or promote products
+ *     derived from this software without specific prior written permission.
  *
  *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -26,9 +28,12 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: devices.h,v 1.110 2004/11/25 10:53:31 debug Exp $
+ *  $Id: devices.h,v 1.131 2005/01/10 23:22:22 debug Exp $
  *
- *  Memory mapped devices:
+ *  Memory mapped devices.
+ *
+ *  TODO: Separate into lots of smaller files? That might speed up a compile,
+ *        but I'm not sure that it's a price worth paying.
  */
 
 #include <sys/types.h>
@@ -43,11 +48,6 @@ struct pci_data;
 #endif */
 
 
-/*  dev_dec5500_ioboard.c:  */
-#define	DEV_DEC5500_IOBOARD_LENGTH		0x100000
-int dev_dec5500_ioboard_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
-struct dec5500_ioboard_data *dev_dec5500_ioboard_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
-
 /*  dev_dec_ioasic.c:  */
 #define	DEV_DEC_IOASIC_LENGTH		0x80100
 #define	N_DEC_IOASIC_REGS	(0x1f0 / 0x10)
@@ -131,16 +131,6 @@ void dev_cons_init(struct memory *mem);
 int dev_colorplanemask_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_colorplanemask_init(struct memory *mem, uint64_t baseaddr, unsigned char *color_plane_mask);
 
-/*  dev_crime.c:  */
-#define	DEV_CRIME_LENGTH		0x0000000000001000
-struct crime_data {
-	unsigned char	reg[DEV_CRIME_LENGTH];
-	int		irq_nr;
-	int		use_fb;
-};
-int dev_crime_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
-struct crime_data *dev_crime_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr, int irq_nr, int use_fb);
-
 /*  dev_dc7085.c:  */
 #define	DEV_DC7085_LENGTH		0x0000000000000080
 /*  see dc7085.h for more info  */
@@ -156,20 +146,14 @@ struct dec5800_data {
 };
 int dev_dec5800_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 struct dec5800_data *dev_dec5800_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
-
-/*  dev_decbi.c:  */
 /*  16 slots, 0x2000 bytes each  */
 #define	DEV_DECBI_LENGTH			0x20000
 int dev_decbi_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_decbi_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
-
-/*  dev_deccca.c:  */
 #define	DEV_DECCCA_LENGTH			0x10000	/*  ?  */
 #define	DEC_DECCCA_BASEADDR			0x19000000	/*  ?  I just made this up  */
 int dev_deccca_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_deccca_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
-
-/*  dev_decxmi.c:  */
 #define	DEV_DECXMI_LENGTH			0x800000
 int dev_decxmi_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_decxmi_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
@@ -235,15 +219,60 @@ void dev_fdc_init(struct memory *mem, uint64_t baseaddr, int irq_nr);
 int dev_gt_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 struct pci_data *dev_gt_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr, int irq_nr, int pciirq);
 
-/*  dev_gt.c:  */
+/*  dev_jazz.c:  */
+#define	DEV_JAZZ_LENGTH			0x280
+struct jazz_data {
+	struct cpu	*cpu;
+
+	/*  Jazz stuff:  */
+	uint32_t	int_enable_mask;
+	uint32_t	int_asserted;
+
+	/*  ISA stuff:  */
+	uint32_t	isa_int_enable_mask;
+	uint32_t	isa_int_asserted;
+
+	int		interval;
+	int		interval_start;
+
+	int		jazz_timer_value;
+	int		jazz_timer_current;
+
+	uint64_t	dma_translation_table_base;
+	uint64_t	dma_translation_table_limit;
+	uint32_t	dma0_mode;
+	uint32_t	dma0_enable;
+	uint32_t	dma0_count;
+	uint32_t	dma0_addr;
+	/*  same for dma1,2,3 actually (TODO)  */
+
+	int		led;
+};
+size_t dev_jazz_dma_controller(void *dma_controller_data,
+	unsigned char *data, size_t len, int writeflag);
+int dev_jazz_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
+struct jazz_data *dev_jazz_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
+
+/*  dev_kn01.c:  */
 #define	DEV_KN01_CSR_LENGTH		0x0000000000000004
 int dev_kn01_csr_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_kn01_csr_init(struct memory *mem, uint64_t baseaddr, int color_fb);
+#define	DEV_VDAC_LENGTH			0x20
+#define	DEV_VDAC_MAPWA			    0x00
+#define	DEV_VDAC_MAP			    0x04
+#define	DEV_VDAC_MASK			    0x08
+#define	DEV_VDAC_MAPRA			    0x0c
+#define	DEV_VDAC_OVERWA			    0x10
+#define	DEV_VDAC_OVER			    0x14
+#define	DEV_VDAC_OVERRA			    0x1c
+int dev_vdac_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
+void dev_vdac_init(struct memory *mem, uint64_t baseaddr, unsigned char *rgb_palette, int color_fb_flag);
 
 /*  dev_kn02.c:  */
-#define	DEV_KN02_LENGTH		0x10
+#define	DEV_KN02_LENGTH		0x1000
 struct kn02_csr {
-	uint32_t	csr;
+	uint8_t		csr[sizeof(uint32_t)];
+	uint8_t		filler[4096 - sizeof(uint32_t)];  /*  for bintrans mapping  */
 };
 int dev_kn02_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 struct kn02_csr *dev_kn02_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
@@ -253,6 +282,14 @@ struct kn02_csr *dev_kn02_init(struct cpu *cpu, struct memory *mem, uint64_t bas
 int dev_kn210_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_kn210_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
 
+/*  dev_kn220.c:  */
+#define	DEV_DEC5500_IOBOARD_LENGTH		0x100000
+int dev_dec5500_ioboard_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
+struct dec5500_ioboard_data *dev_dec5500_ioboard_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
+#define	DEV_SGEC_LENGTH		0x1000
+int dev_sgec_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
+void dev_sgec_init(struct memory *mem, uint64_t baseaddr, int irq_nr);
+
 /*  dev_kn230.c:  */
 #define	DEV_KN230_LENGTH		0x1c00000
 struct kn230_csr {
@@ -282,26 +319,12 @@ void dev_n64_bios_init(struct memory *mem, uint64_t baseaddr);
 int dev_ns16550_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_ns16550_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr, int irq_nr, int addrmult, int in_use);
 
-/*  dev_mace.c:  */
-#define	DEV_MACE_LENGTH			0x100
-struct mace_data {
-	unsigned char	reg[DEV_MACE_LENGTH];
-	int		irqnr;
-};
-int dev_mace_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
-struct mace_data *dev_mace_init(struct memory *mem, uint64_t baseaddr, int irqnr);
-
-/*  dev_macepci.c:  */
-#define	DEV_MACEPCI_LENGTH		0x1000
-int dev_macepci_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
-struct pci_data *dev_macepci_init(struct memory *mem, uint64_t baseaddr, int pciirq);
-
 /*  dev_mc146818.c:  */
 #define	DEV_MC146818_LENGTH		0x0000000000000100
 #define	MC146818_DEC		0
 #define	MC146818_PC_CMOS	1
 #define	MC146818_ARC_NEC	2
-#define	MC146818_ARC_PICA	3
+#define	MC146818_ARC_JAZZ	3
 #define	MC146818_SGI		4
 /*  see mc146818reg.h for more info  */
 void dev_mc146818_tick(struct cpu *cpu, void *);
@@ -316,44 +339,13 @@ void dev_mp_init(struct memory *mem, struct cpu *cpus[]);
 #define	DEV_PCKBC_LENGTH		0x10
 #define	PCKBC_8042		0
 #define	PCKBC_8242		1
-#define	PCKBC_PICA		3
+#define	PCKBC_JAZZ		3
 int dev_pckbc_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_pckbc_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr, int type, int keyboard_irqnr, int mouse_irqnr, int in_use);
 
-/*  dev_pica.c:  */
-#define	DEV_PICA_LENGTH			0x280
-struct pica_data {
-	struct cpu	*cpu;
-
-	/*  Jazz stuff:  */
-	uint32_t	int_enable_mask;
-	uint32_t	int_asserted;
-
-	/*  ISA stuff:  */
-	uint32_t	isa_int_enable_mask;
-	uint32_t	isa_int_asserted;
-
-	int		interval;
-	int		interval_start;
-
-	int		pica_timer_value;
-	int		pica_timer_current;
-
-	uint64_t	dma_translation_table_base;
-	uint64_t	dma_translation_table_limit;
-	uint32_t	dma0_mode;
-	uint32_t	dma0_enable;
-	uint32_t	dma0_count;
-	uint32_t	dma0_addr;
-	/*  same for dma1,2,3 actually (TODO)  */
-};
-size_t dev_pica_dma_controller(void *dma_controller_data,
-	unsigned char *data, size_t len, int writeflag);
-int dev_pica_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
-struct pica_data *dev_pica_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
-
 /*  dev_ps2_gs.c:  */
-#define	DEV_PS2_GIF_LENGTH		0x0000000000010000
+#define	DEV_PS2_GIF_LENGTH		0x10000
+#define	DEV_PS2_GIF_FAKE_BASE		0x50000000
 int dev_ps2_gif_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_ps2_gif_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
 
@@ -384,14 +376,14 @@ struct ps2_data {
 
         uint64_t	dmac_reg[DMAC_REGSIZE / 0x10];
 
-	struct memory	*other_memory[N_PS2_DMA_CHANNELS];
+	uint64_t	other_memory_base[N_PS2_DMA_CHANNELS];
 
 	uint32_t	intr;
 	uint32_t	imask;
 };
 #define	DEV_PS2_STUFF_LENGTH		0x10000
 int dev_ps2_stuff_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
-struct ps2_data *dev_ps2_stuff_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr, struct memory *mem_gif);
+struct ps2_data *dev_ps2_stuff_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
 
 /*  dev_pmagja.c:  */
 #define	DEV_PMAGJA_LENGTH		0x3c0000
@@ -444,11 +436,6 @@ int dev_sfb_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 void dev_sfb_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 	struct vfb_data *vfb_data);
 
-/*  dev_sgec.c:  */
-#define	DEV_SGEC_LENGTH		0x1000
-int dev_sgec_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
-void dev_sgec_init(struct memory *mem, uint64_t baseaddr, int irq_nr);
-
 /*  dev_sgi_cpuinfo.c:  */
 #define	DEV_SGI_CPUINFO_BASE		0x9600000000000000ULL
 #define	DEV_SGI_CPUINFO_LENGTH		0x0001000000000000ULL
@@ -495,27 +482,46 @@ struct sgi_ip22_data *dev_sgi_ip22_init(struct cpu *cpu, struct memory *mem, uin
 int dev_sgi_ip30_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_sgi_ip30_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
 
+/*  dev_sgi_ip32.c:  */
+#define	DEV_CRIME_LENGTH		0x0000000000001000
+struct crime_data {
+	unsigned char	reg[DEV_CRIME_LENGTH];
+	int		irq_nr;
+	int		use_fb;
+};
+int dev_crime_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
+struct crime_data *dev_crime_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr, int irq_nr, int use_fb);
+#define	DEV_MACE_LENGTH			0x100
+struct mace_data {
+	unsigned char	reg[DEV_MACE_LENGTH];
+	int		irqnr;
+};
+int dev_mace_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
+struct mace_data *dev_mace_init(struct memory *mem, uint64_t baseaddr, int irqnr);
+#define	DEV_MACEPCI_LENGTH		0x1000
+int dev_macepci_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
+struct pci_data *dev_macepci_init(struct memory *mem, uint64_t baseaddr, int pciirq);
+#define	DEV_SGI_MEC_LENGTH		0x1000
+int dev_sgi_mec_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
+void dev_sgi_mec_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr, int irq_nr);
+#define	DEV_SGI_UST_LENGTH		0x10000
+int dev_sgi_ust_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
+void dev_sgi_ust_init(struct memory *mem, uint64_t baseaddr);
+#define	DEV_SGI_MTE_LENGTH		0x10000
+int dev_sgi_mte_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
+void dev_sgi_mte_init(struct memory *mem, uint64_t baseaddr);
+
 /*  dev_sgi_mardigras.c:  */
 #define	DEV_SGI_MARDIGRAS_LENGTH	0x800000
 int dev_sgi_mardigras_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_sgi_mardigras_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr);
 
-/*  dev_sgi_mte.c:  */
-#define	DEV_SGI_MTE_LENGTH		0x10000
-int dev_sgi_mte_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
-void dev_sgi_mte_init(struct memory *mem, uint64_t baseaddr);
-
 /*  dev_sgi_nasid.c:  */
 #define	DEV_SGI_NASID_BASE		0x9200000000000000ULL
 #define	DEV_SGI_NASID_LENGTH		0x0000000100000000ULL
 int dev_sgi_nasid_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_sgi_nasid_init(struct memory *mem, uint64_t baseaddr);
 
-/*  dev_sgi_ust.c:  */
-#define	DEV_SGI_UST_LENGTH		0x10000
-int dev_sgi_ust_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
-void dev_sgi_ust_init(struct memory *mem, uint64_t baseaddr);
-
 /*  dev_sii.c:  */
 #define	DEV_SII_LENGTH			0x100
 void dev_sii_tick(struct cpu *cpu, void *);
@@ -545,21 +551,24 @@ void dev_unreadable_init(struct memory *mem, uint64_t baseaddr, uint64_t len);
 void dev_random_init(struct memory *mem, uint64_t baseaddr, uint64_t len);
 void dev_zero_init(struct memory *mem, uint64_t baseaddr, uint64_t len);
 
-/*  dev_vdac.c:  */
-#define	DEV_VDAC_LENGTH			0x20
-#define	DEV_VDAC_MAPWA			    0x00
-#define	DEV_VDAC_MAP			    0x04
-#define	DEV_VDAC_MASK			    0x08
-#define	DEV_VDAC_MAPRA			    0x0c
-#define	DEV_VDAC_OVERWA			    0x10
-#define	DEV_VDAC_OVER			    0x14
-#define	DEV_VDAC_OVERRA			    0x1c
-int dev_vdac_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
-void dev_vdac_init(struct memory *mem, uint64_t baseaddr, unsigned char *rgb_palette, int color_fb_flag);
-
 /*  dev_vga.c:  */
 int dev_vga_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
-void dev_vga_init(struct cpu *cpu, struct memory *mem, uint64_t videomem_base, uint64_t control_base, int max_x, int max_y);
+void dev_vga_init(struct cpu *cpu, struct memory *mem, uint64_t videomem_base, uint64_t control_base, int max_x, int max_y, char *name);
+
+/*  dev_vr41xx.c:  */
+#define	DEV_VR41XX_LENGTH		0x1000		/*  TODO?  */
+struct vr41xx_data {
+	int		cpumodel;
+
+	/*  See icureg.h in NetBSD for more info.  */
+	uint16_t	sysint1;
+	uint16_t	msysint1;
+	uint16_t	sysint2;
+	uint16_t	msysint2;
+};
+
+int dev_vr41xx_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
+struct vr41xx_data *dev_vr41xx_init(struct cpu *cpu, struct memory *mem, int cpumodel);
 
 /*  dev_wdc.c:  */
 #define	DEV_WDC_LENGTH			0x8
