@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -9,6 +9,8 @@
  *  2. Redistributions in binary form must reproduce the above copyright  
  *     notice, this list of conditions and the following disclaimer in the 
  *     documentation and/or other materials provided with the distribution.
+ *  3. The name of the author may not be used to endorse or promote products
+ *     derived from this software without specific prior written permission.
  *
  *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -23,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: bintrans.c,v 1.97 2004/12/01 22:42:29 debug Exp $
+ *  $Id: bintrans.c,v 1.133 2005/01/10 22:30:41 debug Exp $
  *
  *  Dynamic binary translation.
  *
@@ -40,7 +42,7 @@
  *
  *	o)  When the translation cache is "full", then throw away everything
  *	    translated so far and restart from scratch. The cache is of a
- *	    fixed size, say 20 MB. (This is inspired by a comment in the Qemu
+ *	    fixed size, say 24 MB. (This is inspired by a comment in the Qemu
  *	    technical documentation: "A 16 MByte cache holds the most recently
  *	    used translations. For simplicity, it is completely flushed when
  *	    it is full.")
@@ -96,7 +98,11 @@
 #include "misc.h"
 
 #include "bintrans.h"
+#include "cop0.h"
+#include "cpu_types.h"
+#include "emul.h"
 #include "memory.h"
+#include "opcodes.h"
 
 
 #ifndef BINTRANS
@@ -106,8 +112,7 @@
 
 int bintrans_pc_is_in_cache(struct cpu *cpu, uint64_t pc) { return 0; }
 void bintrans_invalidate(struct cpu *cpu, uint64_t paddr) { }
-int bintrans_attempt_translate(struct cpu *cpu, uint64_t paddr,
-	int run_flag) { return 0; }
+int bintrans_attempt_translate(struct cpu *cpu, uint64_t paddr) { return 0; }
 void bintrans_init_cpu(struct cpu *cpu) { }
 void bintrans_init(void)
 {
@@ -122,16 +127,15 @@ void bintrans_init(void)
 static void bintrans_host_cacheinvalidate(unsigned char *p, size_t len);
 static void bintrans_write_chunkreturn(unsigned char **addrp);
 static void bintrans_write_chunkreturn_fail(unsigned char **addrp);
-static void bintrans_write_pc_inc(unsigned char **addrp, int pc_inc,
-	int flag_pc, int flag_ninstr);
-static void bintrans_runchunk(struct cpu *cpu, unsigned char *code);
+static void bintrans_write_pc_inc(unsigned char **addrp);
+/*  static void bintrans_runchunk(struct cpu *cpu, unsigned char *code);  */
 static void bintrans_write_quickjump(unsigned char *quickjump_code, uint32_t chunkoffset);
 static int bintrans_write_instruction__addiu_etc(unsigned char **addrp, int rt, int rs, int imm, int instruction_type);
 static int bintrans_write_instruction__addu_etc(unsigned char **addrp, int rd, int rs, int rt, int sa, int instruction_type);
 static int bintrans_write_instruction__branch(unsigned char **addrp, int instruction_type, int regimm_type, int rt, int rs, int imm);
 static int bintrans_write_instruction__jr(unsigned char **addrp, int rs, int rd, int special);
 static int bintrans_write_instruction__jal(unsigned char **addrp, int imm, int link);
-static int bintrans_write_instruction__delayedbranch(unsigned char **addrp, uint32_t *potential_chunk_p, uint32_t *chunks, int only_care_about_chunk_p, int p);
+static int bintrans_write_instruction__delayedbranch(unsigned char **addrp, uint32_t *potential_chunk_p, uint32_t *chunks, int only_care_about_chunk_p, int p, int forward);
 static int bintrans_write_instruction__loadstore(unsigned char **addrp, int rt, int imm, int rs, int instruction_type, int bigendian);
 static int bintrans_write_instruction__lui(unsigned char **addrp, int rt, int imm);
 static int bintrans_write_instruction__mfmthilo(unsigned char **addrp, int rd, int from_flag, int hi_flag);
@@ -144,18 +148,20 @@ static int bintrans_write_instruction__tlb_rfe_etc(unsigned char **addrp, int it
 #define	TLB_TLBR	3
 #define	TLB_RFE		4
 #define	TLB_ERET	5
+#define	TLB_BREAK	6
+#define	TLB_SYSCALL	7
 
 
-#define	BINTRANS_CACHE_N_INDEX_BITS	14
+#define	BINTRANS_CACHE_N_INDEX_BITS	15
 #define	CACHE_INDEX_MASK		((1 << BINTRANS_CACHE_N_INDEX_BITS) - 1)
 #define	PADDR_TO_INDEX(p)		((p >> 12) & CACHE_INDEX_MASK)
 
 #ifndef BINTRANS_SIZE_IN_MB
-#define BINTRANS_SIZE_IN_MB	20
+#define BINTRANS_SIZE_IN_MB		16
 #endif
 
 #define	CODE_CHUNK_SPACE_SIZE		(BINTRANS_SIZE_IN_MB * 1048576)
-#define	CODE_CHUNK_SPACE_MARGIN		262144
+#define	CODE_CHUNK_SPACE_MARGIN		65536
 
 /*
  *  translation_code_chunk_space is a large chunk of (linear) memory where
@@ -182,10 +188,11 @@ struct translation_page_entry {
 };
 #define	UNTRANSLATABLE		0x01
 
-static struct translation_page_entry **translation_page_entry_array;
+static struct translation_page_entry *translation_page_entry_array[
+    (1 << BINTRANS_CACHE_N_INDEX_BITS) ];
 
 
-#define	MAX_QUICK_JUMPS		15
+#define	MAX_QUICK_JUMPS		8
 static unsigned char *quick_jump_host_address[MAX_QUICK_JUMPS];
 static int quick_jump_page_offset[MAX_QUICK_JUMPS];
 static int n_quick_jumps;
@@ -273,15 +280,37 @@ struct translation_page_entry *prev = NULL;
 }
 
 
+/*
+ *  enter_chunks_into_tables():
+ */
+static void enter_chunks_into_tables(struct cpu *cpu, uint64_t vaddr, uint32_t *chunk0)
+{
+	int a, b;
+	struct vth32_table *tbl1;
+
+	switch (cpu->cpu_type.mmu_model) {
+	case MMU3K:
+		a = (vaddr >> 22) & 0x3ff;
+		b = (vaddr >> 12) & 0x3ff;
+		tbl1 = cpu->vaddr_to_hostaddr_table0_kernel[a];
+		if (tbl1->haddr_entry[b] != NULL)
+			tbl1->bintrans_chunks[b] = chunk0;
+		break;
+	default:
+		;
+	}
+}
+
+
 /*
  *  bintrans_attempt_translate():
  *
  *  Attempt to translate a chunk of code, starting at 'paddr'. If successful,
- *  and "run_flag" is non-zero, then the code chunk is run.
+ *  the code chunk is run.
  *
  *  Returns the number of executed instructions.
  */
-int bintrans_attempt_translate(struct cpu *cpu, uint64_t paddr, int run_flag)
+int bintrans_attempt_translate(struct cpu *cpu, uint64_t paddr)
 {
 	uint64_t paddr_page;
 	int offset_within_page;
@@ -297,18 +326,51 @@ int bintrans_attempt_translate(struct cpu *cpu, uint64_t paddr, int run_flag)
 	unsigned char *f;
 	struct translation_page_entry *tep;
 	size_t chunk_len;
-	int rs,rt=0,rd,sa,imm;
+	int rs,rt,rd,sa,imm;
 	uint32_t *potential_chunk_p;	/*  for branches  */
 	int byte_order_cached_bigendian;
 	int delayed_branch, stop_after_delayed_branch;
 	uint64_t delayed_branch_new_p;
 	int prev_p;
 
-#if 0
-/*  This doesn't work yet.  */
+
+	/*  Abort if the current "environment" isn't safe enough:  */
+	if (cpu->delay_slot || cpu->nullify_next || (paddr & 3) != 0)
+		return cpu->bintrans_instructions_executed;
+
+	bintrans_32bit_only = cpu->cpu_type.mmu_model == MMU3K;
+	byte_order_cached_bigendian = (cpu->byte_order == EMUL_BIG_ENDIAN);
+
+	/*  Is this a part of something that is already translated?  */
+	paddr_page = paddr & ~0xfff;
+	offset_within_page = (paddr & 0xfff) / 4;
+	entry_index = PADDR_TO_INDEX(paddr);
+	tep = translation_page_entry_array[entry_index];
+	while (tep != NULL) {
+		if (tep->paddr == paddr_page) {
+			if (tep->chunk[offset_within_page] != 0) {
+				f = (size_t)tep->chunk[offset_within_page] +
+				    translation_code_chunk_space;
+				goto run_it;	/*  see further down  */
+			}
+			if (tep->flags[offset_within_page] & UNTRANSLATABLE)
+				return cpu->bintrans_instructions_executed;
+			break;
+		}
+		tep = tep->next;
+	}
+
+#if 1
+/*  printf("A paddr=%016llx\n", (long long)paddr);  */
+/*  Sometimes this works.  */
 quick_attempt_translate_again:
 #endif
-
+/*printf("B: ");
+printf("v=%016llx p=%016llx h=%p paddr=%016llx\n",
+(long long)cpu->pc_last_virtual_page,
+(long long)cpu->pc_last_physical_page,
+cpu->pc_last_host_4k_page,(long long)paddr);
+*/
 	/*
 	 *  If the chunk space is all used up, we need to start over from
 	 *  an empty chunk space.
@@ -319,36 +381,12 @@ quick_attempt_translate_again:
 			translation_page_entry_array[i] = NULL;
 		translation_code_chunk_space_head = 0;
 		n_quick_jumps = 0;
+		tep = NULL;
 		debug("bintrans: Starting over!\n");
+		clear_all_chunks_from_all_tables(cpu);
 	}
 
 
-	/*  Abort if the current "environment" isn't safe enough:  */
-	if (cpu->delay_slot || cpu->nullify_next || (paddr & 3) != 0)
-		return cpu->bintrans_instructions_executed;
-
-	/*  Is this a part of something that is already translated?  */
-	paddr_page = paddr & ~0xfff;
-	offset_within_page = (paddr & 0xfff) / 4;
-	entry_index = PADDR_TO_INDEX(paddr);
-	tep = translation_page_entry_array[entry_index];
-	while (tep != NULL) {
-		if (tep->paddr == paddr_page) {
-			if (tep->flags[offset_within_page] & UNTRANSLATABLE)
-				return cpu->bintrans_instructions_executed;
-			if (tep->chunk[offset_within_page] != 0) {
-				if (!run_flag)
-					return cpu->bintrans_instructions_executed;
-
-				f = (size_t)tep->chunk[offset_within_page] +
-				    translation_code_chunk_space;
-				goto run_it;	/*  see further down  */
-			}
-			break;
-		}
-		tep = tep->next;
-	}
-
 	host_mips_page = cpu->pc_bintrans_host_4kpage;
 	if (host_mips_page == NULL)
 		return cpu->bintrans_instructions_executed;
@@ -362,7 +400,7 @@ quick_attempt_translate_again:
 
 		/*  ... and align again:  */
 		translation_code_chunk_space_head =
-		    ((translation_code_chunk_space_head - 1) | 31) + 1;
+		    ((translation_code_chunk_space_head - 1) | 63) + 1;
 
 		/*  Add the entry to the array:  */
 		memset(tep, 0, sizeof(struct translation_page_entry));
@@ -383,15 +421,11 @@ quick_attempt_translate_again:
 	 *  Make sure that this page will not be written to by translated
 	 *  code:
 	 */
-	update_translation_table(cpu, 0x80000000 + paddr, NULL, -1, 0);
-	update_translation_table(cpu, 0xa0000000 + paddr, NULL, -1, 0);
-
+	invalidate_translation_caches_paddr(cpu, paddr);
 
 	/*
 	 *  Try to translate a chunk of code:
 	 */
-	bintrans_32bit_only = cpu->cpu_type.mmu_model == MMU3K;
-	byte_order_cached_bigendian = (cpu->byte_order == EMUL_BIG_ENDIAN);
 	p = paddr & 0xfff;
 	try_to_translate = 1;
 	n_translated = 0;
@@ -399,7 +433,7 @@ quick_attempt_translate_again:
 	delayed_branch = 0;
 	stop_after_delayed_branch = 0;
 	delayed_branch_new_p = 0;
-
+	rt = 0;
 	n_quick_jumps = quick_jumps_index = 0;
 
 	while (try_to_translate) {
@@ -437,6 +471,13 @@ quick_attempt_translate_again:
 				if (special6 == SPECIAL_JR)
 					stop_after_delayed_branch = 1;
 				break;
+			case SPECIAL_SYSCALL:
+			case SPECIAL_BREAK:
+				translated = bintrans_write_instruction__tlb_rfe_etc(&ca,
+				    special6 == SPECIAL_BREAK? TLB_BREAK : TLB_SYSCALL);
+				n_translated += translated;
+ 				try_to_translate = 0;
+				break;
 			case SPECIAL_ADDU:
 			case SPECIAL_DADDU:
 			case SPECIAL_SUBU:
@@ -461,6 +502,8 @@ quick_attempt_translate_again:
 			case SPECIAL_SLTU:
 			case SPECIAL_MOVZ:
 			case SPECIAL_MOVN:
+			case SPECIAL_DIV:
+			case SPECIAL_DIVU:
 			case SPECIAL_MULT:
 			case SPECIAL_MULTU:
 			case SPECIAL_SYNC:
@@ -528,9 +571,13 @@ quick_attempt_translate_again:
 			break;
 
 		case HI6_BEQ:
+		case HI6_BEQL:
 		case HI6_BNE:
+		case HI6_BNEL:
 		case HI6_BLEZ:
+		case HI6_BLEZL:
 		case HI6_BGTZ:
+		case HI6_BGTZL:
 			rs = ((instr[3] & 3) << 3) + ((instr[2] >> 5) & 7);
 			rt = instr[2] & 31;
 			imm = (instr[1] << 8) + instr[0];
@@ -542,12 +589,14 @@ quick_attempt_translate_again:
 			delayed_branch_new_p = p + 4 + 4*imm;
 			break;
 
+		case HI6_ADDI:
 		case HI6_ADDIU:
 		case HI6_SLTI:
 		case HI6_SLTIU:
 		case HI6_ANDI:
 		case HI6_ORI:
 		case HI6_XORI:
+		case HI6_DADDI:
 		case HI6_DADDIU:
 			rs = ((instr[3] & 3) << 3) + ((instr[2] >> 5) & 7);
 			rt = instr[2] & 31;
@@ -618,17 +667,25 @@ quick_attempt_translate_again:
 				try_to_translate = 0;
 			break;
 
+#if 0
 		case HI6_LQ_MDMX:
+#endif
 		case HI6_LD:
 		case HI6_LWU:
 		case HI6_LW:
+		case HI6_LWL:
+		case HI6_LWR:
 		case HI6_LHU:
 		case HI6_LH:
 		case HI6_LBU:
 		case HI6_LB:
+#if 0
 		case HI6_SQ:
+#endif
 		case HI6_SD:
 		case HI6_SW:
+		case HI6_SWL:
+		case HI6_SWR:
 		case HI6_SH:
 		case HI6_SB:
 			rs = ((instr[3] & 3) << 3) + ((instr[2] >> 5) & 7);
@@ -636,17 +693,7 @@ quick_attempt_translate_again:
 			imm = (instr[1] << 8) + instr[0];
 			if (imm >= 32768)
 				imm -= 65536;
-#if 0
-switch (cpu->cpu_type.mmu_model) {
-case MMU3K:
-#endif
 			translated = try_to_translate = bintrans_write_instruction__loadstore(&ca, rt, imm, rs, hi6, byte_order_cached_bigendian);
-#if 0
-	break;
-default:
-	translated = try_to_translate = 0;
-}
-#endif
 			n_translated += translated;
 			break;
 
@@ -666,6 +713,8 @@ default:
 		if (translated && delayed_branch) {
 			delayed_branch --;
 			if (delayed_branch == 0) {
+				int forward;
+
 				/*
 				 *  p is 0x000 .. 0xffc. If the jump is to
 				 *  within the same page, then we can use
@@ -678,9 +727,11 @@ default:
 				else
 					potential_chunk_p = NULL;
 
+				forward = delayed_branch_new_p > p;
+
 				bintrans_write_instruction__delayedbranch(&ca,
 				    potential_chunk_p, &tep->chunk[0], 0,
-				    delayed_branch_new_p & 0xfff);
+				    delayed_branch_new_p & 0xfff, forward);
 
 				if (stop_after_delayed_branch)
 					try_to_translate = 0;
@@ -709,12 +760,21 @@ default:
 		    prev_p < 1023 && tep->chunk[prev_p+1] != 0
 		    && !delayed_branch) {
 			bintrans_write_instruction__delayedbranch(
-			    &ca, &tep->chunk[prev_p+1], NULL, 1, prev_p+1);
+			    &ca, &tep->chunk[prev_p+1], NULL, 1, p+4, 1);
 			try_to_translate = 0;
 		}
 
-		if (n_translated > 150)
+		if (translated && try_to_translate && n_translated > 80
+		    && prev_p < 1023 && !delayed_branch) {
+			bintrans_write_instruction__delayedbranch(
+			    &ca, &tep->chunk[prev_p+1], NULL, 1, p+4, 1);
 			try_to_translate = 0;
+		}
+
+		if (translated && try_to_translate && tep->flags[prev_p+1] & UNTRANSLATABLE
+		    && prev_p < 1023 && !delayed_branch) {
+			bintrans_write_chunkreturn_fail(&ca);
+		}
 
 		p += sizeof(instr);
 
@@ -741,25 +801,25 @@ default:
 	/*  chunk_len = nr of bytes occupied by the new code chunk  */
 	chunk_len = (size_t)ca - (size_t)ca2;
 
-	/*  Invalidate the host's instruction cache, if neccessary:  */
+	/*  Invalidate the host's instruction cache, if necessary:  */
 	bintrans_host_cacheinvalidate(ca2, chunk_len);
 
 	translation_code_chunk_space_head += chunk_len;
 
 	/*  Align the code chunk space:  */
 	translation_code_chunk_space_head =
-	    ((translation_code_chunk_space_head - 1) | 31) + 1;
-
-	if (!run_flag)
-		return cpu->bintrans_instructions_executed;
+	    ((translation_code_chunk_space_head - 1) | 63) + 1;
 
 
 	/*  RUN the code chunk:  */
 	f = ca2;
+
 run_it:
 	/*  printf("BEFORE: pc=%016llx r31=%016llx\n",
 	    (long long)cpu->pc, (long long)cpu->gpr[31]); */
 
+	enter_chunks_into_tables(cpu, cpu->pc, &tep->chunk[0]);
+
 	old_n_executed = cpu->bintrans_instructions_executed;
 
 	bintrans_runchunk(cpu, f);
@@ -771,13 +831,12 @@ run_it:
 	    cpu->bintrans_instructions_executed < N_SAFE_BINTRANS_LIMIT
 	    && (cpu->pc & 3) == 0
 	    && cpu->bintrans_instructions_executed != old_n_executed) {
-		uint64_t paddr;
-		int ok = 0;
-		int a, b;
+		int ok = 0, a, b;
 		struct vth32_table *tbl1;
 
-		switch (cpu->cpu_type.mmu_model) {
-		case MMU3K:
+		if (bintrans_32bit_only ||
+		    (cpu->pc & 0xffffffff80000000ULL) == 0 ||
+		    (cpu->pc & 0xffffffff80000000ULL) == 0xffffffff80000000ULL) {
 			/*  32-bit special case:  */
 			a = (cpu->pc >> 22) & 0x3ff;
 			b = (cpu->pc >> 12) & 0x3ff;
@@ -804,8 +863,9 @@ run_it:
 			ok = cpu->translate_address(cpu, cpu->pc, &paddr, FLAG_INSTR);
 
 			if (!ok && old_pc != cpu->pc) {
-				ok = cpu->translate_address(cpu, cpu->pc, &paddr,
-				    FLAG_INSTR + FLAG_NOEXCEPTIONS);
+				/*  pc is something like ...0080 or ...0000 or so.  */
+				paddr = cpu->pc & 0xfff;
+				ok = 1;
 
 				cpu->pc_last_host_4k_page = NULL;
 				cpu->pc_bintrans_host_4kpage = NULL;
@@ -822,35 +882,54 @@ run_it:
 					if (tep->chunk[offset_within_page] != 0) {
 						f = (size_t)tep->chunk[offset_within_page] +
 						    translation_code_chunk_space;
-						goto run_it;	/*  see further down  */
+						goto run_it;
 					}
+					if (tep->flags[offset_within_page] & UNTRANSLATABLE)
+						return cpu->bintrans_instructions_executed;
 					break;
 				}
 				tep = tep->next;
 			}
 
-			/*  We have no translation. This special hack
-			    might make the time spent in the main cpu_run_instr()
-			    lower:  */
-			switch (cpu->cpu_type.mmu_model) {
-			case MMU3K:
+#if 1
+			/*  We have no translation.  */
+			if ((cpu->pc & 0xfff00000) == 0xbfc00000 &&
+			    cpu->emul->prom_emulation)
+				return cpu->bintrans_instructions_executed;
+
+			/*  This special hack might make the time spent
+			    in the main cpu_run_instr() lower:  */
+			if (bintrans_32bit_only) {
+			/*  TODO: This doesn't seem to work with R4000 etc?
+			    || (cpu->pc & 0xffffffff80000000ULL) == 0 ||
+			    (cpu->pc & 0xffffffff80000000ULL) == 0xffffffff80000000ULL) {  */
+				int ok = 1;
 				/*  32-bit special case:  */
 				a = (cpu->pc >> 22) & 0x3ff;
 				b = (cpu->pc >> 12) & 0x3ff;
+				if (cpu->vaddr_to_hostaddr_table0 !=
+				    cpu->vaddr_to_hostaddr_table0_kernel)
+					ok = 0;
 				tbl1 = cpu->vaddr_to_hostaddr_table0_kernel[a];
-				if (tbl1->haddr_entry[b] != NULL) {
+				if (ok && tbl1->haddr_entry[b] != NULL) {
 					cpu->pc_last_virtual_page = cpu->pc & ~0xfff;
 					cpu->pc_last_physical_page = paddr & ~0xfff;
 					cpu->pc_last_host_4k_page = (unsigned char *)
 					    (((size_t)tbl1->haddr_entry[b]) & ~1);
 					cpu->pc_bintrans_host_4kpage = cpu->pc_last_host_4k_page;
 					cpu->pc_bintrans_paddr = paddr;
-#if 0
-					/*  Why doesn't this work? TODO  */
+
+/*
+printf("C: ");
+printf("v=%016llx p=%016llx h=%p paddr=%016llx\n",
+(long long)cpu->pc_last_virtual_page,
+(long long)cpu->pc_last_physical_page,
+cpu->pc_last_host_4k_page,(long long)paddr);
+*/
 					goto quick_attempt_translate_again;
-#endif
 				}
 			}
+#endif
 
 			/*  Return.  */
 		}
@@ -871,62 +950,59 @@ void bintrans_init_cpu(struct cpu *cpu)
 {
 	int i, offset;
 
-	cpu->chunk_base_address   = translation_code_chunk_space;
-	cpu->bintrans_fast_tlbwri = coproc_tlbwri;
-	cpu->bintrans_fast_tlbpr  = coproc_tlbpr;
-	cpu->bintrans_fast_rfe    = coproc_rfe;
-	cpu->bintrans_fast_eret   = coproc_eret;
-
-	cpu->fast_vaddr_to_hostaddr = fast_vaddr_to_hostaddr;
+	cpu->chunk_base_address        = translation_code_chunk_space;
+	cpu->bintrans_loadstore_32bit  = bintrans_loadstore_32bit;
+	cpu->bintrans_jump_to_32bit_pc = bintrans_jump_to_32bit_pc;
+	cpu->bintrans_fast_tlbwri      = coproc_tlbwri;
+	cpu->bintrans_fast_tlbpr       = coproc_tlbpr;
+	cpu->bintrans_fast_rfe         = coproc_rfe;
+	cpu->bintrans_fast_eret        = coproc_eret;
+	cpu->bintrans_simple_exception = cpu_cause_simple_exception;
+	cpu->fast_vaddr_to_hostaddr    = fast_vaddr_to_hostaddr;
 
 	/*  Initialize vaddr->hostaddr translation tables:  */
-	switch (cpu->cpu_type.mmu_model) {
-	case MMU3K:
-		cpu->vaddr_to_hostaddr_nulltable =
-		    zeroed_alloc(sizeof(struct vth32_table));
+	cpu->vaddr_to_hostaddr_nulltable =
+	    zeroed_alloc(sizeof(struct vth32_table));
 
-		/*  Data cache:  */
-		offset = 0;
-		cpu->vaddr_to_hostaddr_r2k3k_dcachetable =
-		    zeroed_alloc(sizeof(struct vth32_table));
-		for (i=0; i<1024; i++) {
-			cpu->vaddr_to_hostaddr_r2k3k_dcachetable->haddr_entry[i] =
-			    (void *)(((size_t)cpu->cache[0]+offset) | 1);
-			offset = (offset + 4096) % cpu->cache_size[0];
-		}
-		cpu->vaddr_to_hostaddr_r2k3k_dcachetable->refcount = 1024;
-
-		/*  Instruction cache:  */
-		offset = 0;
-		cpu->vaddr_to_hostaddr_r2k3k_icachetable =
-		    zeroed_alloc(sizeof(struct vth32_table));
-		for (i=0; i<1024; i++) {
-			cpu->vaddr_to_hostaddr_r2k3k_icachetable->haddr_entry[i] =
-			    (void *)(((size_t)cpu->cache[1]+offset) | 1);
-			offset = (offset + 4096) % cpu->cache_size[1];
-		}
-		cpu->vaddr_to_hostaddr_r2k3k_icachetable->refcount = 1024;
-
-		cpu->vaddr_to_hostaddr_table0_kernel =
-		    zeroed_alloc(1024 * sizeof(struct vth32_table *));
-		cpu->vaddr_to_hostaddr_table0_user =
-		    zeroed_alloc(1024 * sizeof(struct vth32_table *));
-		cpu->vaddr_to_hostaddr_table0_cacheisol_i =
-		    zeroed_alloc(1024 * sizeof(struct vth32_table *));
-		cpu->vaddr_to_hostaddr_table0_cacheisol_d =
-		    zeroed_alloc(1024 * sizeof(struct vth32_table *));
-
-		for (i=0; i<1024; i++) {
-			cpu->vaddr_to_hostaddr_table0_kernel[i] = cpu->vaddr_to_hostaddr_nulltable;
-			cpu->vaddr_to_hostaddr_table0_user[i] = cpu->vaddr_to_hostaddr_nulltable;
-			cpu->vaddr_to_hostaddr_table0_cacheisol_i[i] = cpu->vaddr_to_hostaddr_r2k3k_icachetable;
-			cpu->vaddr_to_hostaddr_table0_cacheisol_d[i] = cpu->vaddr_to_hostaddr_r2k3k_dcachetable;
-		}
-
-		cpu->vaddr_to_hostaddr_table0 = cpu->vaddr_to_hostaddr_table0_kernel;
-
-		break;
+	/*  Data cache:  */
+	offset = 0;
+	cpu->vaddr_to_hostaddr_r2k3k_dcachetable =
+	    zeroed_alloc(sizeof(struct vth32_table));
+	for (i=0; i<1024; i++) {
+		cpu->vaddr_to_hostaddr_r2k3k_dcachetable->haddr_entry[i] =
+		    (void *)(((size_t)cpu->cache[0]+offset) | 1);
+		offset = (offset + 4096) % cpu->cache_size[0];
 	}
+	cpu->vaddr_to_hostaddr_r2k3k_dcachetable->refcount = 1024;
+
+	/*  Instruction cache:  */
+	offset = 0;
+	cpu->vaddr_to_hostaddr_r2k3k_icachetable =
+	    zeroed_alloc(sizeof(struct vth32_table));
+	for (i=0; i<1024; i++) {
+		cpu->vaddr_to_hostaddr_r2k3k_icachetable->haddr_entry[i] =
+		    (void *)(((size_t)cpu->cache[1]+offset) | 1);
+		offset = (offset + 4096) % cpu->cache_size[1];
+	}
+	cpu->vaddr_to_hostaddr_r2k3k_icachetable->refcount = 1024;
+
+	cpu->vaddr_to_hostaddr_table0_kernel =
+	    zeroed_alloc(1024 * sizeof(struct vth32_table *));
+	cpu->vaddr_to_hostaddr_table0_user =
+	    zeroed_alloc(1024 * sizeof(struct vth32_table *));
+	cpu->vaddr_to_hostaddr_table0_cacheisol_i =
+	    zeroed_alloc(1024 * sizeof(struct vth32_table *));
+	cpu->vaddr_to_hostaddr_table0_cacheisol_d =
+	    zeroed_alloc(1024 * sizeof(struct vth32_table *));
+
+	for (i=0; i<1024; i++) {
+		cpu->vaddr_to_hostaddr_table0_kernel[i] = cpu->vaddr_to_hostaddr_nulltable;
+		cpu->vaddr_to_hostaddr_table0_user[i] = cpu->vaddr_to_hostaddr_nulltable;
+		cpu->vaddr_to_hostaddr_table0_cacheisol_i[i] = cpu->vaddr_to_hostaddr_r2k3k_icachetable;
+		cpu->vaddr_to_hostaddr_table0_cacheisol_d[i] = cpu->vaddr_to_hostaddr_r2k3k_dcachetable;
+	}
+
+	cpu->vaddr_to_hostaddr_table0 = cpu->vaddr_to_hostaddr_table0_kernel;
 }
 
 
@@ -942,28 +1018,12 @@ void bintrans_init(void)
 
 	debug("bintrans: EXPERIMENTAL!\n");
 
-	s = 1 << BINTRANS_CACHE_N_INDEX_BITS;
-	s *= sizeof(struct translation_page_entry *);
-	translation_page_entry_array = (void *) mmap(NULL, s,
-	    PROT_READ | PROT_WRITE, MAP_ANON | MAP_PRIVATE, -1, 0);
-	if (translation_page_entry_array == NULL) {
-		translation_page_entry_array = malloc(s);
-		if (translation_page_entry_array == NULL) {
-			fprintf(stderr, "bintrans_init(): out of memory (1)\n");
-			exit(1);
-		}
-
-		/*
-		 *  The entry array must be NULLed, as these are pointers to
-		 *  translation page entries. If the mmap() succeeded, then
-		 *  the array is zero-filled by default anyway...
-		 */
-		for (i=0; i<n; i++)
-			translation_page_entry_array[i] = NULL;
-	}
-
-	debug("bintrans: translation_page_entry_array = %i KB at %p\n",
-	    (int)(s/1024), translation_page_entry_array);
+	/*
+	 *  The entry array must be NULLed, as these are pointers to
+	 *  translation page entries.
+	 */
+	for (i=0; i<n; i++)
+		translation_page_entry_array[i] = NULL;
 
 	/*  Allocate the large code chunk space:  */
 	s = CODE_CHUNK_SPACE_SIZE + CODE_CHUNK_SPACE_MARGIN;
