@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -9,6 +9,8 @@
  *  2. Redistributions in binary form must reproduce the above copyright  
  *     notice, this list of conditions and the following disclaimer in the 
  *     documentation and/or other materials provided with the distribution.
+ *  3. The name of the author may not be used to endorse or promote products
+ *     derived from this software without specific prior written permission.
  *
  *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -23,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: diskimage.c,v 1.53 2004/11/30 21:47:44 debug Exp $
+ *  $Id: diskimage.c,v 1.63 2005/01/09 01:55:30 debug Exp $
  *
  *  Disk image support.
  *
@@ -48,37 +50,13 @@
 #include "misc.h"
 
 #include "diskimage.h"
-
-
-struct diskimage {
-	char		*fname;
-	off_t		total_size;
-
-	int		writable;
-	int		is_a_cdrom;
-	int		is_boot_device;
-
-	int		logical_block_size;
-
-	int		is_a_tape;
-	uint64_t	tape_offset;
-	int		tape_filenr;
-	int		filemark;
-
-	int		rpms;
-	int		ncyls;
-
-	FILE		*f;
-};
+#include "emul.h"
 
 
 extern int quiet_mode;
 
-
-#define	MAX_DISKIMAGES		8
-
-static struct diskimage *diskimages[MAX_DISKIMAGES];
-static int n_diskimages = 0;
+struct diskimage *diskimages[MAX_DISKIMAGES];
+int n_diskimages = 0;
 
 
 static struct scsi_transfer *first_free_scsi_transfer_alloc = NULL;
@@ -338,7 +316,7 @@ static void diskimage__switch_tape(int disk_id)
  *
  *  The xferp points to a scsi_transfer struct, containing msg_out, command,
  *  and data_out coming from the SCSI controller device.  This function
- *  interprets the command, and (if neccessary) creates responses in
+ *  interprets the command, and (if necessary) creates responses in
  *  data_in, msg_in, and status.
  *
  *  Returns:
@@ -372,7 +350,11 @@ int diskimage_scsicommand(struct cpu *cpu, int disk_id,
 	debug("[ diskimage_scsicommand(id=%i) cmd=0x%02x: ",
 	    disk_id, xferp->cmd[0]);
 
-	/*  fatal("[ diskimage_scsicommand(id=%i) cmd=0x%02x ]\n", disk_id, xferp->cmd[0]);  */
+#if 0
+	fatal("[ diskimage_scsicommand(id=%i) cmd=0x%02x ]\n",
+	    disk_id, xferp->cmd[0]);
+#endif
+
 #if 0
 {
 	static FILE *f = NULL;
@@ -466,6 +448,11 @@ xferp->data_in[4] = 0x2c - 4;	/*  Additional length  */
 				memcpy(xferp->data_in+8,  "DEC     ", 8);
 				memcpy(xferp->data_in+16, "RRD42   (C) DEC ", 16);
 				memcpy(xferp->data_in+32, "4.5d", 4);
+			} else {
+				/*  NEC, CD-ROM:  */
+				memcpy(xferp->data_in+8,  "NEC     ", 8);
+				memcpy(xferp->data_in+16, "CD-ROM CDR-210P ", 16);
+				memcpy(xferp->data_in+32, "1.0 ", 4);
 			}
 		}
 
@@ -780,7 +767,7 @@ xferp->data_in[4] = 0x2c - 4;	/*  Additional length  */
 		diskimage_access(disk_id, 1, ofs, xferp->data_out, size);
 		/*  TODO: how about return code?  */
 
-		/*  Is this really neccessary?  */
+		/*  Is this really necessary?  */
 		/*  fsync(fileno(diskimages[disk_id]->f));  */
 
 		diskimage__return_default_status_and_message(xferp);
@@ -997,8 +984,12 @@ printf(" XXX \n");
 		break;
 
 	case SCSICDROM_READ_TOC:
-		retlen = 12;
-		debug("CDROM_READ_TOC: TODO");
+		debug("(CDROM_READ_TOC: ");
+		debug("lun=%i msf=%i ",
+		    xferp->cmd[1] >> 5, (xferp->cmd[1] >> 1) & 1);
+		debug("starting_track=%i ", xferp->cmd[6]);
+		retlen = xferp->cmd[7] * 256 + xferp->cmd[8];
+		debug("allocation_len=%i)\n", retlen);
 
 		/*  Return data:  */
 		scsi_transfer_allocbuf(&xferp->data_in_len,
@@ -1065,6 +1056,20 @@ printf(" XXX \n");
 		diskimage__return_default_status_and_message(xferp);
 		break;
 
+	case 0xbd:
+		fatal("[ SCSI 0x%02x: TODO ]\n", xferp->cmd[0]);
+
+		/*
+		 *  Used by Windows NT?
+		 *
+		 *  Not documented in http://www.danbbs.dk/~dino/SCSI/SCSI2-D.html.
+		 *
+		 *  TODO
+		 */
+
+		diskimage__return_default_status_and_message(xferp);
+		break;
+
 	default:
 		fatal("unimplemented SCSI command 0x%02x, disk id=%i\n", xferp->cmd[0], disk_id);
 		exit(1);
@@ -1389,6 +1394,20 @@ int diskimage_bootdev(void)
 }
 
 
+/*
+ *  diskimage_is_a_cdrom():
+ *
+ *  Returns 1 if a disk image is a SCSI CDROM, 0 otherwise.
+ */
+int diskimage_is_a_cdrom(int i)
+{
+	if (i<0 || diskimages[i] == NULL)
+		return 0;
+
+	return diskimages[i]->is_a_cdrom;
+}
+
+
 /*
  *  diskimage_is_a_tape():
  *
@@ -1398,7 +1417,7 @@ int diskimage_bootdev(void)
  */
 int diskimage_is_a_tape(int i)
 {
-	if (diskimages[i] == NULL)
+	if (i<0 || diskimages[i] == NULL)
 		return 0;
 
 	return diskimages[i]->is_a_tape;
