@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -9,6 +9,8 @@
  *  2. Redistributions in binary form must reproduce the above copyright  
  *     notice, this list of conditions and the following disclaimer in the 
  *     documentation and/or other materials provided with the distribution.
+ *  3. The name of the author may not be used to endorse or promote products
+ *     derived from this software without specific prior written permission.
  *
  *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -23,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: main.c,v 1.133 2004/11/29 14:14:55 debug Exp $
+ *  $Id: main.c,v 1.149 2005/01/09 01:55:31 debug Exp $
  */
 
 #include <stdio.h>
@@ -35,6 +37,7 @@
 
 #include "misc.h"
 
+#include "cpu_types.h"
 #include "diskimage.h"
 #include "emul.h"
 
@@ -101,14 +104,14 @@ static void usage(char *progname, int longusage)
 	int i;
 	struct cpu_type_def cpu_type_defs[] = CPU_TYPE_DEFS;
 
-	printf("mips64emul-%s  Copyright (C) 2003-2004  Anders Gavare\n",
+	printf("mips64emul-%s  Copyright (C) 2003-2005  Anders Gavare\n",
 #ifdef VERSION
 	    VERSION
 #else
 	    "(no version)"
 #endif
 	    );
-	printf("Read the documentation and/or source code for other copyright notices.\n");
+	printf("Read the documentation and/or source code for other Copyright notices.\n");
 	printf("\nusage: %s [options] file [...]\n", progname);
 
 	if (!longusage) {
@@ -118,7 +121,8 @@ static void usage(char *progname, int longusage)
 
 	printf("\n");
 	printf("  -A x      try to emulate an ARC machine (1=NEC-RD94, 2=PICA-61, 3=NEC-R94,\n");
-	printf("            4=Deskstation Tyne, 5=MIPS Magnum, 6=NEC-R98, 7=Olivetti M700)\n");
+	printf("            4=Deskstation Tyne, 5=MIPS Magnum, 6=NEC-R98, 7=Olivetti M700,\n");
+	printf("            8=NEC-R96)\n");
 	printf("  -a        emulate a generic test machine\n");
 	printf("  -B        try to emulate a Playstation 2 machine\n");
 #ifdef BINTRANS
@@ -150,7 +154,8 @@ static void usage(char *progname, int longusage)
 	printf("                0-7   force a specific SCSI ID number\n");
 	printf("  -E        try to emulate a Cobalt machine\n");
 	printf("  -e        try to emulate a MeshCube\n");
-	printf("  -F        try to emulate an hpcmips machine\n");
+	printf("  -F xx     try to emulate an hpcmips machine, where 'xx' may be:\n");
+	printf("                1=Casio BE-300, 2=Casio E-105\n");
 	printf("  -f        try to emulate a Sony NeWS MIPS machine\n");
 	printf("  -G xx     try to emulate an SGI machine, IPxx\n");
 	printf("  -g        try to emulate a NetGear box (WG602)\n");
@@ -168,13 +173,11 @@ static void usage(char *progname, int longusage)
 	printf("  -m nr     run at most nr instructions (on any cpu)\n");
 	printf("  -N        display nr of instructions/second average, at regular intervals\n");
 	printf("  -n nr     set nr of CPUs\n");
-	printf("  -O        fake netboot (tftp instead of rzX), even when a disk image is\n"
-	       "            present (for DECstation emulation)\n");
+	printf("  -O        fake netboot (tftp instead of disk), even when a disk image is\n"
+	       "            present (for DECstation, SGI, and ARC emulation)\n");
 	printf("  -o arg    set the boot argument (for DEC, ARC, or SGI emulation).\n");
 	printf("            Default arg for DEC is '-a', for ARC '-aN'.\n");
-	printf("  -P pc     add a PC dumppoint.  (if the PC register ever holds this value,\n");
-	printf("            register dumping (-r) and instruction trace (-i) are enabled)\n");
-	printf("  -p pc     same as -P, but only enables -i, not -r\n");
+	printf("  -p pc     add a breakpoint (remember to use the '0x' prefix for hex!)\n");
 	printf("  -Q        no built-in PROM emulation  (use this for running ROM images)\n");
 	printf("  -q        quiet mode (don't print startup or debug messages)\n");
 	printf("  -R        use random bootstrap cpu, instead of nr 0\n");
@@ -183,10 +186,11 @@ static void usage(char *progname, int longusage)
 	printf("  -s        show opcode usage statistics after simulation\n");
 	printf("  -T        start -i and -r traces on accesses to invalid memory addresses\n");
 	printf("  -t        show function trace tree\n");
-	printf("  -U        dump TLB entries when the TLB is used for lookups\n");
+	printf("  -U        enable slow_serial_interrupts_hack_for_linux\n");
 #ifdef ENABLE_USERLAND
 	printf("  -u x      userland-only (syscall) emulation; 1=NetBSD/pmax, 2=Ultrix/pmax\n");
 #endif
+	printf("  -V        start up in the single-step debugger, paused\n");
 	printf("  -v        verbose debug messages\n");
 #ifdef WITH_X11
 	printf("  -X        use X11\n");
@@ -225,7 +229,7 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul)
 
 	symbol_init(&emul->symbol_context);
 
-	while ((ch = getopt(argc, argv, "A:aBbC:D:d:EeFfG:gHhI:iJj:M:m:Nn:Oo:P:p:QqRrSsTtUu:vXY:y:Z:z:")) != -1) {
+	while ((ch = getopt(argc, argv, "A:aBbC:D:d:EeF:fG:gHhI:iJj:M:m:Nn:Oo:p:QqRrSsTtUu:VvXY:y:Z:z:")) != -1) {
 		switch (ch) {
 		case 'A':
 			emul->emulation_type = EMULTYPE_ARC;
@@ -264,7 +268,7 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul)
 			break;
 		case 'F':
 			emul->emulation_type = EMULTYPE_HPCMIPS;
-			emul->machine = 0;
+			emul->machine = atoi(optarg);
 			break;
 		case 'f':
 			emul->emulation_type = EMULTYPE_SONYNEWS;
@@ -328,21 +332,20 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul)
 			strcpy(emul->boot_string_argument, optarg);
 			using_switch_o = 1;
 			break;
-		case 'P':
 		case 'p':
-			if (emul->n_dumppoints >= MAX_PC_DUMPPOINTS) {
-				fprintf(stderr, "too many pc dumppoints\n");
+			if (emul->n_breakpoints >= MAX_BREAKPOINTS) {
+				fprintf(stderr, "too many breakpoints\n");
 				exit(1);
 			}
-			emul->dumppoint_string[emul->n_dumppoints] =
+			emul->breakpoint_string[emul->n_breakpoints] =
 			    malloc(strlen(optarg) + 1);
-			if (emul->dumppoint_string[emul->n_dumppoints] == NULL) {
+			if (emul->breakpoint_string[emul->n_breakpoints] == NULL) {
 				fprintf(stderr, "out of memory\n");
 				exit(1);
 			}
-			strcpy(emul->dumppoint_string[emul->n_dumppoints], optarg);
-			emul->dumppoint_flag_r[emul->n_dumppoints] = (ch == 'P') ? 1 : 0;
-			emul->n_dumppoints ++;
+			strcpy(emul->breakpoint_string[emul->n_breakpoints], optarg);
+			emul->breakpoint_flags[emul->n_breakpoints] = 0;
+			emul->n_breakpoints ++;
 			break;
 		case 'Q':
 			emul->prom_emulation = 0;
@@ -369,11 +372,14 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul)
 			emul->show_trace_tree = 1;
 			break;
 		case 'U':
-			emul->tlb_dump = 1;
+			emul->slow_serial_interrupts_hack_for_linux = 1;
 			break;
 		case 'u':
 			emul->userland_emul = atoi(optarg);
 			break;
+		case 'V':
+			emul->single_step = 1;
+			break;
 		case 'v':
 			emul->verbose ++;
 			break;
@@ -444,6 +450,10 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul)
 	/*  Default nr of CPUs (for SMP systems):  */
 
 	if (!n_cpus_set) {
+		if (emul->emulation_type == EMULTYPE_ARC &&
+		    emul->machine == MACHINE_ARC_NEC_R96)
+			emul->ncpus = 2;
+
 		if (emul->emulation_type == EMULTYPE_ARC &&
 		    emul->machine == MACHINE_ARC_NEC_R98)
 			emul->ncpus = 4;
@@ -470,6 +480,18 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul)
 	    && !emul->emul_cpu_name[0])
 		strcpy(emul->emul_cpu_name, "R3000");
 
+	if (emul->emulation_type == EMULTYPE_HPCMIPS
+	    && !emul->emul_cpu_name[0]) {
+		if (emul->machine == HPCMIPS_CASIO_BE300)
+			strcpy(emul->emul_cpu_name, "VR4131");
+		else if (emul->machine == HPCMIPS_CASIO_E105)
+			strcpy(emul->emul_cpu_name, "VR4121");
+		else {
+			printf("Unimplemented HPCMIPS model?\n");
+			exit(1);
+		}
+	}
+
 	if (emul->emulation_type == EMULTYPE_COBALT && !emul->emul_cpu_name[0])
 		strcpy(emul->emul_cpu_name, "RM5200");
 
@@ -529,6 +551,18 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul)
 	if (emul->emulation_type == EMULTYPE_SGI && emul->physical_ram_in_mb == 0)
 		emul->physical_ram_in_mb = 48;
 
+	if (emul->emulation_type == EMULTYPE_HPCMIPS &&
+	    emul->physical_ram_in_mb == 0) {
+		switch (emul->machine) {
+		case HPCMIPS_CASIO_BE300:
+			emul->physical_ram_in_mb = 16;
+			break;
+		case HPCMIPS_CASIO_E105:
+			emul->physical_ram_in_mb = 32;
+			break;
+		}
+	}
+
 	if (emul->emulation_type == EMULTYPE_MESHCUBE && emul->physical_ram_in_mb == 0)
 		emul->physical_ram_in_mb = 64;
 
@@ -646,6 +680,11 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul)
 		exit(1);
 	}
 
+	if (emul->n_breakpoints > 0 && emul->bintrans_enable) {
+		fprintf(stderr, "Breakpoints and dynamic translation don't work too well together right now.\n");
+		exit(1);
+	}
+
 #ifndef BINTRANS
 	if (emul->bintrans_enable) {
 		fprintf(stderr, "WARNING: %s was compiled without bintrans support. Ignoring -b.\n", progname);
@@ -670,6 +709,8 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul)
 	if (!using_switch_Z && !emul->use_x11)
 		emul->n_gfx_cards = 0;
 
+	emul->bintrans_enabled_from_start = emul->bintrans_enable;
+
 	return 0;
 }
 
