@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -9,6 +9,8 @@
  *  2. Redistributions in binary form must reproduce the above copyright  
  *     notice, this list of conditions and the following disclaimer in the 
  *     documentation and/or other materials provided with the distribution.
+ *  3. The name of the author may not be used to endorse or promote products
+ *     derived from this software without specific prior written permission.
  *
  *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -23,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: memory.c,v 1.122 2004/12/01 14:57:45 debug Exp $
+ *  $Id: memory.c,v 1.138 2005/01/12 14:07:22 debug Exp $
  *
  *  Functions for handling the memory of an emulated machine.
  */
@@ -38,6 +40,7 @@
 #include "misc.h"
 
 #include "bintrans.h"
+#include "cop0.h"
 #include "memory.h"
 
 
@@ -67,7 +70,7 @@ uint64_t memory_readmax64(struct cpu *cpu, unsigned char *buf, int len)
 	}
 #endif
 
-	/*  Switch byte order for incoming data, if neccessary:  */
+	/*  Switch byte order for incoming data, if necessary:  */
 	if (cpu->byte_order == EMUL_BIG_ENDIAN)
 		for (i=0; i<len; i++) {
 			x <<= 8;
@@ -465,7 +468,7 @@ int memory_cache_R3000(struct cpu *cpu, int cache, uint64_t paddr,
 			memblock = memory_paddr_to_hostaddr(
 			    mem, old_cached_paddr, MEM_WRITE);
 			offset = old_cached_paddr
-			    & (mem->memblock_size - 1)
+			    & ((1 << BITS_PER_MEMBLOCK) - 1)
 			    & ~cpu->cache_mask[which_cache];
 
 			src = cpu->cache[which_cache];
@@ -491,7 +494,7 @@ int memory_cache_R3000(struct cpu *cpu, int cache, uint64_t paddr,
 
 		/*  Copy from main memory into the cache:  */
 		memblock = memory_paddr_to_hostaddr(mem, paddr, writeflag);
-		offset = paddr & (mem->memblock_size - 1)
+		offset = paddr & ((1 << BITS_PER_MEMBLOCK) - 1)
 		    & ~cpu->cache_mask[which_cache];
 		/*  offset is offset within the memblock:
 		 *  printf("write: offset = 0x%x\n", offset);
@@ -543,7 +546,7 @@ int memory_cache_R3000(struct cpu *cpu, int cache, uint64_t paddr,
 		memblock = memory_paddr_to_hostaddr(mem, paddr, writeflag);
 		if (memblock != NULL) {
 			cpu->pc_last_host_4k_page = memblock +
-			    (paddr & (mem->memblock_size - 1) & ~0xfff);
+			    (paddr & ((1 << BITS_PER_MEMBLOCK) - 1) & ~0xfff);
 		}
 	}
 
@@ -612,7 +615,7 @@ int memory_cache_R3000(struct cpu *cpu, int cache, uint64_t paddr,
 		}
 		return 1;
 	} else {
-		/*  Reload caches if neccessary:  */
+		/*  Reload caches if necessary:  */
 
 		/*  No!  Not when not emulating caches fully. (TODO?)  */
 		cpu->cache_last_paddr[cache] = paddr;
@@ -629,6 +632,18 @@ int memory_cache_R3000(struct cpu *cpu, int cache, uint64_t paddr,
 #undef TRANSLATE_ADDRESS
 #undef V2P_MMU3K
 
+#define TRANSLATE_ADDRESS	translate_address_mmu8k
+#define	V2P_MMU8K
+#include "memory_v2p.c"
+#undef TRANSLATE_ADDRESS
+#undef V2P_MMU8K
+
+#define TRANSLATE_ADDRESS	translate_address_mmu10k
+#define	V2P_MMU10K
+#include "memory_v2p.c"
+#undef TRANSLATE_ADDRESS
+#undef V2P_MMU10K
+
 #define TRANSLATE_ADDRESS	translate_address_generic
 #include "memory_v2p.c"
 
@@ -668,13 +683,15 @@ int memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	uint64_t paddr;
 	int cache, no_exceptions, ok = 1, offset;
 	unsigned char *memblock;
-
+#ifdef BINTRANS
+	int bintrans_cached = cpu->emul->bintrans_enable;
+#endif
 	no_exceptions = cache_flags & NO_EXCEPTIONS;
 	cache = cache_flags & CACHE_FLAGS_MASK;
 
 
 #ifdef BINTRANS
-	if (cpu->emul->bintrans_enable) {
+	if (bintrans_cached) {
 		if (cache == CACHE_INSTRUCTION) {
 			cpu->pc_bintrans_host_4kpage = NULL;
 			cpu->pc_bintrans_paddr_valid = 0;
@@ -739,7 +756,7 @@ have_paddr:
 
 	/*  TODO: How about bintrans vs cache emulation?  */
 #ifdef BINTRANS
-	if (cpu->emul->bintrans_enable) {
+	if (bintrans_cached) {
 		if (cache == CACHE_INSTRUCTION) {
 			cpu->pc_bintrans_paddr_valid = 1;
 			cpu->pc_bintrans_paddr = paddr;
@@ -782,21 +799,21 @@ into the devices  */
 					len = mem->dev_length[i] - paddr;
 
 #ifdef BINTRANS
-				if (mem->dev_flags[i] & MEM_BINTRANS_OK) {
+				if (bintrans_cached && mem->dev_flags[i] & MEM_BINTRANS_OK) {
 					int wf = writeflag == MEM_WRITE? 1 : 0;
 
-					if (cpu->emul->bintrans_enable && writeflag) {
-						if (paddr < cpu->mem->dev_bintrans_write_low[i])
-						    cpu->mem->dev_bintrans_write_low[i] = paddr & ~0xfff;
-						if (paddr > cpu->mem->dev_bintrans_write_high[i])
-						    cpu->mem->dev_bintrans_write_high[i] = paddr | 0xfff;
+					if (writeflag) {
+						if (paddr < mem->dev_bintrans_write_low[i])
+						    mem->dev_bintrans_write_low[i] = paddr & ~0xfff;
+						if (paddr > mem->dev_bintrans_write_high[i])
+						    mem->dev_bintrans_write_high[i] = paddr | 0xfff;
 					}
 
 					if (!(mem->dev_flags[i] & MEM_BINTRANS_WRITE_OK))
 						wf = 0;
 
 					update_translation_table(cpu, vaddr & ~0xfff,
-					    cpu->mem->dev_bintrans_data[i] + (paddr & ~0xfff),
+					    mem->dev_bintrans_data[i] + (paddr & ~0xfff),
 					    wf, orig_paddr & ~0xfff);
 				}
 #endif
@@ -836,35 +853,16 @@ into the devices  */
 	}
 
 
-#if 0
-	{
-		/*  Accesses that cross memblocks are bad:  */
-		uint64_t endaddr = paddr + len - 1;
-		endaddr &= ~(mem->memblock_size - 1);
-		if ( (paddr & ~(mem->memblock_size - 1)) != endaddr ) {
-			fatal("memory access crosses memory block? "
-			    "paddr=%016llx len=%i\n", (long long)paddr,
-			    (int)len);
-			return MEMORY_ACCESS_FAILED;
-		}
-	}
-#endif
-
-
-
 	/*
 	 *  Data and instruction cache emulation:
 	 */
-	/*  vaddr shouldn't be used below anyway,
-	    except for in the next 'if' statement:  */
-	if ((vaddr >> 32) == 0xffffffffULL)
-		vaddr &= 0xffffffffULL;
 
 	switch (cpu->cpu_type.mmu_model) {
 	case MMU3K:
 		/*  if not uncached addess  (TODO: generalize this)  */
 		if (!(cache_flags & PHYSICAL) && cache != CACHE_NONE &&
-		    !(vaddr >= 0xa0000000ULL && vaddr <= 0xbfffffffULL)) {
+		    !((vaddr & 0xffffffffULL) >= 0xa0000000ULL &&
+		      (vaddr & 0xffffffffULL) <= 0xbfffffffULL)) {
 			if (memory_cache_R3000(cpu, cache, paddr,
 			    writeflag, len, data))
 				goto do_return_ok;
@@ -923,7 +921,7 @@ into the devices  */
 				uint64_t offset;
 
 				if (!quiet_mode) {
-					debug("[ memory_rw(): writeflag=%i ", writeflag);
+					fatal("[ memory_rw(): writeflag=%i ", writeflag);
 					if (writeflag) {
 						unsigned int i;
 						debug("data={", writeflag);
@@ -934,7 +932,7 @@ into the devices  */
 					symbol = get_symbol_name(
 					    &cpu->emul->symbol_context,
 					    cpu->pc_last, &offset);
-fatal(" paddr=%llx >= physical_max pc=0x%08llx <%s> ]\n",
+					fatal(" paddr=%llx >= physical_max pc=0x%08llx <%s> ]\n",
 					    (long long)paddr, (long long)cpu->pc_last, symbol? symbol : "no symbol");
 				}
 
@@ -985,14 +983,19 @@ no_exception_access:
 
 	offset = paddr & ((1 << BITS_PER_MEMBLOCK) - 1);
 
-	if (cpu->emul->bintrans_enable)
+#ifdef BINTRANS
+	if (bintrans_cached)
 		update_translation_table(cpu, vaddr & ~0xfff,
 		    memblock + (offset & ~0xfff),
+#if 0
 		    cache == CACHE_INSTRUCTION?
 			(writeflag == MEM_WRITE? 1 : 0)
 			: ok - 1,
-/*		    writeflag == MEM_WRITE? 1 : 0,  */
+#else
+		    writeflag == MEM_WRITE? 1 : 0,
+#endif
 		    paddr & ~0xfff);
+#endif
 
 	if (writeflag == MEM_WRITE) {
 		if (len == sizeof(uint32_t) && (offset & 3)==0)
@@ -1013,7 +1016,7 @@ no_exception_access:
 			cpu->pc_last_host_4k_page = memblock
 			    + (offset & ~0xfff);
 #ifdef BINTRANS
-			if (cpu->emul->bintrans_enable) {
+			if (bintrans_cached) {
 				cpu->pc_bintrans_host_4kpage =
 				    cpu->pc_last_host_4k_page;
 			}
@@ -1067,10 +1070,10 @@ void memory_device_bintrans_access(struct cpu *cpu, struct memory *mem, void *ex
 
 			for (s=0; s<mem->dev_length[i]; s+=4096) {
 				update_translation_table(cpu,
-				    mem->dev_baseaddr[i] + s + 0x80000000ULL,
+				    mem->dev_baseaddr[i] + s + 0xffffffff80000000ULL,
 				    mem->dev_bintrans_data[i] + s, -1, mem->dev_baseaddr[i] + s);
 				update_translation_table(cpu,
-				    mem->dev_baseaddr[i] + s + 0xa0000000ULL,
+				    mem->dev_baseaddr[i] + s + 0xffffffffa0000000ULL,
 				    mem->dev_bintrans_data[i] + s, -1, mem->dev_baseaddr[i] + s);
 			}
 
@@ -1083,6 +1086,31 @@ void memory_device_bintrans_access(struct cpu *cpu, struct memory *mem, void *ex
 }
 
 
+/*
+ *  memory_device_register_statefunction():
+ *
+ *  TODO: Hm. This is semi-ugly. Should probably be rewritten/redesigned
+ *  some day.
+ */
+void memory_device_register_statefunction(
+	struct memory *mem, void *extra,
+	int (*dev_f_state)(struct cpu *,
+	    struct memory *, void *extra, int wf, int nr,
+	    int *type, char **namep, void **data, size_t *len))
+{
+	int i;
+
+	for (i=0; i<mem->n_mmapped_devices; i++)
+		if (mem->dev_extra[i] == extra) {
+			mem->dev_f_state[i] = dev_f_state;
+			return;
+		}
+
+	printf("memory_device_register_statefunction(): couldn't find the device\n");
+	exit(1);
+}
+
+
 /*
  *  memory_device_register():
  *
@@ -1100,7 +1128,7 @@ void memory_device_register(struct memory *mem, const char *device_name,
 		exit(1);
 	}
 
-	debug("adding device %i at 0x%016llx: %s\n",
+	debug("adding device %2i at 0x%016llx: %s\n",
 	    mem->n_mmapped_devices, (long long)baseaddr, device_name);
 
 	mem->dev_name[mem->n_mmapped_devices] = device_name;
@@ -1129,5 +1157,3 @@ void memory_device_register(struct memory *mem, const char *device_name,
 		mem->mmap_dev_maxaddr = baseaddr + len;
 }
 
-
-
