@@ -9,6 +9,8 @@
  *  2. Redistributions in binary form must reproduce the above copyright  
  *     notice, this list of conditions and the following disclaimer in the 
  *     documentation and/or other materials provided with the distribution.
+ *  3. The name of the author may not be used to endorse or promote products
+ *     derived from this software without specific prior written permission.
  *
  *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -23,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: net.c,v 1.33 2004/11/03 00:36:49 debug Exp $
+ *  $Id: net.c,v 1.42 2005/01/09 01:55:31 debug Exp $
  *
  *  Emulated (ethernet / internet) network support.
  *
@@ -40,8 +42,8 @@
  *		o)  remove the netbsd-specific options in the tcp header (?)
  *		o)  Outgoing UDP packet fragment support.
  *		o)  IPv6
- *		o)  Make sure it works with Sprite too
  *		o)  DHCP? (Just return the 10.x.x.x default values.)
+ *		o)  Incoming connections
  *
  *
  *  The emulated NIC has a MAC address of (for example) 10:20:30:40:50:60.
@@ -108,8 +110,8 @@ static struct in_addr nameserver_ipv4;
 
 static int64_t net_timestamp = 0;
 
-#define	MAX_TCP_CONNECTIONS	40
-#define	MAX_UDP_CONNECTIONS	15
+#define	MAX_TCP_CONNECTIONS	60
+#define	MAX_UDP_CONNECTIONS	60
 
 struct udp_connection {
 	int		in_use;
@@ -141,7 +143,7 @@ struct tcp_connection {
 	uint32_t	inside_timestamp;
 
 	/*  TODO:  tx and rx buffers?  */
-	unsigned char	incoming_buf[2000];
+	unsigned char	*incoming_buf;
 	int		incoming_buf_rounds;
 	int		incoming_buf_len;
 	uint32_t	incoming_buf_seqnr;
@@ -160,6 +162,8 @@ struct tcp_connection {
 	uint32_t	outside_timestamp;
 };
 
+#define	TCP_INCOMING_BUF_LEN	2000
+
 #define	TCP_OUTSIDE_TRYINGTOCONNECT	1
 #define	TCP_OUTSIDE_CONNECTED		2
 #define	TCP_OUTSIDE_DISCONNECTED	3
@@ -627,7 +631,7 @@ static void net_ip_tcp(void *extra, unsigned char *packet, int len)
 	 *  TODO:  Send back RST?
 	 */
 	if (con_id < 0 && !syn) {
-		fatal("[ net: TCP: dropping packet from unknown connection, %i.%i.%i.%i:%i -> %i.%i.%i.%i:%i %s%s%s%s%s]\n",
+		debug("[ net: TCP: dropping packet from unknown connection, %i.%i.%i.%i:%i -> %i.%i.%i.%i:%i %s%s%s%s%s]\n",
 		    packet[26], packet[27], packet[28], packet[29], srcport,
 		    packet[30], packet[31], packet[32], packet[33], dstport,
 		    fin? "FIN ": "", syn? "SYN ": "", ack? "ACK ": "",
@@ -1062,7 +1066,7 @@ static void net_ip(void *extra, unsigned char *packet, int len)
 /*
  *  net_arp():
  *
- *  Handle an ARP packet, coming from the emulated NIC.
+ *  Handle an ARP (or RARP) packet, coming from the emulated NIC.
  *
  *  An ARP packet might look like this:
  *
@@ -1080,12 +1084,15 @@ static void net_ip(void *extra, unsigned char *packet, int len)
  *  An ARP request with the same from and to IP addresses should be ignored.
  *  (This would be a host testing to see if there is an IP collision.)
  */
-static void net_arp(void *extra, unsigned char *packet, int len)
+static void net_arp(void *extra, unsigned char *packet, int len, int reverse)
 {
 	int i;
 
 	/*  TODO: This debug dump assumes ethernet->IPv4 translation:  */
-	debug("[ net: ARP: ");
+	if (reverse)
+		debug("[ net: RARP: ");
+	else
+		debug("[ net: ARP: ");
 	for (i=0; i<2; i++)
 		debug("%02x", packet[i]);
 	debug(" ");
@@ -1135,8 +1142,35 @@ static void net_arp(void *extra, unsigned char *packet, int len)
 			lp->data[6 + 14] = 0x00; lp->data[7 + 14] = 0x02;
 
 			break;
-		case 2:		/*  Reply  */
 		case 3:		/*  Reverse Request  */
+			lp = net_allocate_packet_link(extra, len + 14);
+
+			/*  Copy the old packet first:  */
+			memcpy(lp->data + 14, packet, len);
+
+			/*  Add ethernet RARP header:  */
+			memcpy(lp->data + 0, packet + 8, 6);
+			memcpy(lp->data + 6, gateway_addr, 6);
+			lp->data[12] = 0x80; lp->data[13] = 0x35;
+
+			/*  This is a RARP reply:  */
+			lp->data[6 + 14] = 0x00; lp->data[7 + 14] = 0x04;
+
+			/*  Address of the gateway:  */
+			memcpy(lp->data +  8 + 14, gateway_addr, 6);
+			memcpy(lp->data + 14 + 14, gateway_ipv4, 4);
+
+			/*  MAC address of emulated machine:  */
+			memcpy(lp->data + 18 + 14, packet + 8, 6);
+
+			/*  IP address of the emulated machine:  */
+			lp->data[24 + 14] = 10;
+			lp->data[25 + 14] =  0;
+			lp->data[26 + 14] =  0;
+			lp->data[27 + 14] =  1;
+
+			break;
+		case 2:		/*  Reply  */
 		case 4:		/*  Reverse Reply  */
 		default:
 			fatal("[ net: ARP: UNIMPLEMENTED request type 0x%04x ]\n", r);
@@ -1245,7 +1279,7 @@ int net_ethernet_rx_avail(void *extra)
 		udp_data[7] = 0;
 		memcpy(udp_data + 8, buf, res);
 		/*
-		 *  TODO:  UDP checksum, if neccessary. At least NetBSD
+		 *  TODO:  UDP checksum, if necessary. At least NetBSD
 		 *  and OpenBSD accept UDP packets with 0x0000 in the
 		 *  checksum field anyway.
 		 */
@@ -1331,6 +1365,14 @@ int net_ethernet_rx_avail(void *extra)
 			continue;
 		}
 
+		if (tcp_connections[con_id].incoming_buf == NULL) {
+			tcp_connections[con_id].incoming_buf = malloc(TCP_INCOMING_BUF_LEN);
+			if (tcp_connections[con_id].incoming_buf == NULL) {
+				printf("out of memory allocating incoming_buf for con_id %i\n", con_id);
+				exit(1);
+			}
+		}
+
 		if (tcp_connections[con_id].state >=
 		    TCP_OUTSIDE_DISCONNECTED)
 			continue;
@@ -1510,7 +1552,7 @@ int net_ethernet_rx(void *extra, unsigned char **packetp, int *lenp)
  *  net_ethernet_tx():
  *
  *  Transmit an ethernet packet, as seen from the emulated ethernet controller.
- *  If the packet can be handled here, it will not neccessarily be transmitted
+ *  If the packet can be handled here, it will not necessarily be transmitted
  *  to the outside world.
  */
 void net_ethernet_tx(void *extra, unsigned char *packet, int len)
@@ -1554,7 +1596,7 @@ void net_ethernet_tx(void *extra, unsigned char *packet, int len)
 			if (packet[i] == 0xff)
 				n++;
 		if (n == 6) {
-			fatal("[ net: TX: IP broadcast: TODO ]\n");
+			debug("[ net: TX: IP broadcast: TODO ]\n");
 			return;
 		}
 
@@ -1566,15 +1608,16 @@ void net_ethernet_tx(void *extra, unsigned char *packet, int len)
 	}
 
 	/*  ARP:  */
-	if (len == 60 && packet[12] == 0x08 && packet[13] == 0x06) {
-		net_arp(extra, packet + 14, len - 14);
+	if (packet[12] == 0x08 && packet[13] == 0x06) {
+		if (len != 60)
+			fatal("[ net_ethernet_tx: WARNING! unusual ARP len (%i) ]\n", len);
+		net_arp(extra, packet + 14, len - 14, 0);
 		return;
 	}
 
 	/*  RARP:  */
 	if (packet[12] == 0x80 && packet[13] == 0x35) {
-		/*  TODO.  */
-		fatal("[ net: TX: UNIMPLEMENTED RARP packet ]\n");
+		net_arp(extra, packet + 14, len - 14, 1);
 		return;
 	}
 
