@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -9,6 +9,8 @@
  *  2. Redistributions in binary form must reproduce the above copyright  
  *     notice, this list of conditions and the following disclaimer in the 
  *     documentation and/or other materials provided with the distribution.
+ *  3. The name of the author may not be used to endorse or promote products
+ *     derived from this software without specific prior written permission.
  *
  *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -23,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: useremul.c,v 1.20 2004/10/17 15:31:45 debug Exp $
+ *  $Id: useremul.c,v 1.24 2005/01/12 08:45:49 debug Exp $
  *
  *  Userland (syscall) emulation.
  *
@@ -65,6 +67,7 @@
 #include <time.h>
 #include <errno.h>
 
+#include "emul.h"
 #include "misc.h"
 
 
@@ -221,7 +224,7 @@ static unsigned char *get_userland_buf(struct cpu *cpu,
  *  Handle userland syscalls.  This function is called whenever
  *  a userland process runs a 'syscall' instruction.  The code
  *  argument is the code embedded into the syscall instruction.
- *  (This 'code' value is not neccessarily used by specific
+ *  (This 'code' value is not necessarily used by specific
  *  emulations.)
  */
 void useremul_syscall(struct cpu *cpu, uint32_t code)
